"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Description = exports.Response = exports.Parameters = exports.Trace = exports.Accept = exports.Delete = exports.Patch = exports.Put = exports.Post = exports.Get = exports.ControllerMethod = exports.Tags = exports.Controller = exports.methodsMetaKey = exports.controllerMetaKey = void 0;
require("reflect-metadata");
const Errors_1 = require("../Errors");
const lodash_1 = require("lodash");
exports.controllerMetaKey = Symbol('Controller');
exports.methodsMetaKey = Symbol('Method');
function Controller(path) {
    return function (target) {
        registerControllerProperty(target, { basePath: path });
    };
}
exports.Controller = Controller;
function Tags(...args) {
    return function (target) {
        registerControllerProperty(target, { tags: args });
    };
}
exports.Tags = Tags;
function ControllerMethod(method, path) {
    return function (target, propertyKey, descriptor) {
        registerMethodProperty(target, { propertyKey, method, path: path || '' });
        let originalMethod = descriptor.value;
        descriptor.value = async function (request) {
            try {
                const response = await originalMethod.apply(this, [request]);
                if (response && lodash_1.get(response, 'statusCode')) {
                    return {
                        statusCode: response.statusCode,
                        body: response.body ?? '',
                    };
                }
                if (response && typeof response === 'object') {
                    return {
                        statusCode: 200,
                        body: response.body,
                    };
                }
                if (typeof response === 'string') {
                    return {
                        statusCode: 200,
                        body: response,
                    };
                }
            }
            catch (e) {
                if (e instanceof Errors_1.BaseError) {
                    return {
                        statusCode: parseInt(e.statusCode),
                        body: e.context || { error: e.message },
                    };
                }
                else {
                    throw e;
                }
            }
        };
    };
}
exports.ControllerMethod = ControllerMethod;
function Get(path) {
    const processedPath = !path || path.trim() === '/' ? undefined : path;
    return ControllerMethod('GET', processedPath);
}
exports.Get = Get;
function Post(path) {
    const processedPath = !path || path.trim() === '/' ? undefined : path;
    return ControllerMethod('POST', processedPath);
}
exports.Post = Post;
function Put(path) {
    const processedPath = !path || path.trim() === '/' ? undefined : path;
    return ControllerMethod('PUT', processedPath);
}
exports.Put = Put;
function Patch(path) {
    const processedPath = !path || path.trim() === '/' ? undefined : path;
    return ControllerMethod('PATCH', processedPath);
}
exports.Patch = Patch;
function Delete(path) {
    const processedPath = !path || path.trim() === '/' ? undefined : path;
    return ControllerMethod('DELETE', processedPath);
}
exports.Delete = Delete;
function Accept(...contentTypes) {
    return function (_target, _propertyKey, descriptor) {
        let originalMethod = descriptor.value;
        descriptor.value = async function (request) {
            const sentContentType = request.headers.getHeader('Content-type');
            if (!sentContentType || !contentTypes.includes(sentContentType)) {
                return {
                    statusCode: 415,
                    body: { error: 'Unsupported media type' },
                };
            }
            return originalMethod.apply(this, [request]);
        };
    };
}
exports.Accept = Accept;
function Trace(params) {
    const logRequest = Boolean(params?.logRequest);
    const logResponse = Boolean(params?.logResponse);
    return function (target, propertyKey) {
        registerMethodProperty(target, {
            propertyKey,
            logRequest,
            logResponse,
        });
    };
}
exports.Trace = Trace;
function Parameters(schemas) {
    return function (target, propertyKey, descriptor) {
        registerMethodProperty(target, { propertyKey, parameters: schemas });
        let originalMethod = descriptor.value;
        descriptor.value = async function (request) {
            const { body, headers, path, query, multiValueQuery } = request;
            const { pathParametersSchema, queryStringParametersSchema, headersSchema, bodySchema, multiValueQueryStringSchema, } = schemas;
            if (bodySchema && (!body || typeof body !== 'object')) {
                throw new Errors_1.BadRequest('Malformed JSON body');
            }
            try {
                if (pathParametersSchema)
                    await pathParametersSchema.validateAsync(path, { allowUnknown: true });
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In path: ${String(e)}`);
            }
            try {
                if (queryStringParametersSchema)
                    await queryStringParametersSchema.validateAsync(query, { allowUnknown: true });
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In querystring: ${String(e)}`);
            }
            try {
                if (multiValueQueryStringSchema)
                    await multiValueQueryStringSchema.validateAsync(multiValueQuery, { allowUnknown: true });
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In querystring: ${String(e)}`);
            }
            try {
                if (headersSchema)
                    await headersSchema.validateAsync(headers);
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In header: ${String(e)}`);
            }
            try {
                if (bodySchema)
                    await bodySchema.validateAsync(body, { allowUnknown: false });
            }
            catch (e) {
                throw new Errors_1.BadRequest(`In body: ${String(e)}`);
            }
            return originalMethod.apply(this, [request]);
        };
    };
}
exports.Parameters = Parameters;
function Response(code, _description, examples) {
    return function (target, propertyKey) {
        registerMethodProperty(target, {
            propertyKey,
            responseExamples: examples ? { [String(code)]: examples } : undefined,
        });
    };
}
exports.Response = Response;
function Description(description) {
    return function (target, propertyKey) {
        registerMethodProperty(target, { propertyKey, description });
    };
}
exports.Description = Description;
function registerControllerProperty(target, meta) {
    const properties = Reflect.getMetadata(exports.controllerMetaKey, target.prototype) || {};
    const newProperties = { ...properties, ...meta };
    Reflect.defineMetadata(exports.controllerMetaKey, newProperties, target.prototype);
}
function registerMethodProperty(target, meta) {
    const properties = Reflect.getMetadata(exports.methodsMetaKey, target) || {};
    const currentProperties = properties[meta.propertyKey] || {};
    if (meta.responseExamples) {
        currentProperties.responseExamples = currentProperties.responseExamples || {};
        for (const [code, exampleList] of Object.entries(meta.responseExamples)) {
            currentProperties.responseExamples[code] = currentProperties.responseExamples[code] || [];
            currentProperties.responseExamples[code].push(...exampleList);
        }
    }
    const newProperties = { ...currentProperties, ...meta };
    properties[meta.propertyKey] = newProperties;
    Reflect.defineMetadata(exports.methodsMetaKey, properties, target);
}
